@@ -4,28 +4,26 @@
 
 """Check for signs of poor design."""
 
+from __future__ import annotations
+
 import re
-import sys
 from collections import defaultdict
-from typing import TYPE_CHECKING, FrozenSet, Iterator, List, Set, cast
+from collections.abc import Iterator
+from typing import TYPE_CHECKING
 
 import astroid
 from astroid import nodes
 
-from pylint import utils
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import check_messages
-from pylint.interfaces import IAstroidChecker
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from astroid.decorators import cachedproperty as cached_property
+from pylint.checkers.utils import is_enum, only_required_for_messages
+from pylint.typing import MessageDefinitionTuple
 
 if TYPE_CHECKING:
     from pylint.lint import PyLinter
 
-MSGS = {  # pylint: disable=consider-using-namedtuple-or-dataclass
+MSGS: dict[
+    str, MessageDefinitionTuple
+] = {  # pylint: disable=consider-using-namedtuple-or-dataclass
     "R0901": (
         "Too many ancestors (%s/%s)",
         "too-many-ancestors",
@@ -177,7 +175,7 @@ def _is_exempt_from_public_methods(node: astroid.ClassDef) -> bool:
 
     # If it's a typing.Namedtuple, typing.TypedDict or an Enum
     for ancestor in node.ancestors():
-        if ancestor.name == "Enum" and ancestor.root().name == "enum":
+        if is_enum(ancestor):
             return True
         if ancestor.qname() in (TYPING_NAMEDTUPLE, TYPING_TYPEDDICT):
             return True
@@ -204,7 +202,7 @@ def _is_exempt_from_public_methods(node: astroid.ClassDef) -> bool:
     return False
 
 
-def _count_boolean_expressions(bool_op):
+def _count_boolean_expressions(bool_op: nodes.BoolOp) -> int:
     """Counts the number of boolean expressions in BoolOp `bool_op` (recursive).
 
     example: a and (b or c or (d and e)) ==> 5 boolean expressions
@@ -218,7 +216,7 @@ def _count_boolean_expressions(bool_op):
     return nb_bool_expr
 
 
-def _count_methods_in_class(node):
+def _count_methods_in_class(node: nodes.ClassDef) -> int:
     all_methods = sum(1 for method in node.methods() if not method.name.startswith("_"))
     # Special methods count towards the number of public methods,
     # but don't count towards there being too many methods.
@@ -229,7 +227,7 @@ def _count_methods_in_class(node):
 
 
 def _get_parents_iter(
-    node: nodes.ClassDef, ignored_parents: FrozenSet[str]
+    node: nodes.ClassDef, ignored_parents: frozenset[str]
 ) -> Iterator[nodes.ClassDef]:
     r"""Get parents of ``node``, excluding ancestors of ``ignored_parents``.
 
@@ -246,8 +244,8 @@ def _get_parents_iter(
     And ``ignored_parents`` is ``{"E"}``, then this function will return
     ``{A, B, C, D}`` -- both ``E`` and its ancestors are excluded.
     """
-    parents: Set[nodes.ClassDef] = set()
-    to_explore = cast(List[nodes.ClassDef], list(node.ancestors(recurs=False)))
+    parents: set[nodes.ClassDef] = set()
+    to_explore = list(node.ancestors(recurs=False))
     while to_explore:
         parent = to_explore.pop()
         if parent.qname() in ignored_parents:
@@ -265,8 +263,8 @@ def _get_parents_iter(
 
 
 def _get_parents(
-    node: nodes.ClassDef, ignored_parents: FrozenSet[str]
-) -> Set[nodes.ClassDef]:
+    node: nodes.ClassDef, ignored_parents: frozenset[str]
+) -> set[nodes.ClassDef]:
     return set(_get_parents_iter(node, ignored_parents))
 
 
@@ -278,13 +276,10 @@ class MisdesignChecker(BaseChecker):
     * size, complexity of functions, methods
     """
 
-    __implements__ = (IAstroidChecker,)
-
     # configuration section name
     name = "design"
     # messages
     msgs = MSGS
-    priority = -2
     # configuration options
     options = (
         (
@@ -403,31 +398,27 @@ class MisdesignChecker(BaseChecker):
         ),
     )
 
-    def __init__(self, linter=None):
+    def __init__(self, linter: PyLinter) -> None:
         super().__init__(linter)
-        self._returns = None
-        self._branches = None
-        self._stmts = None
+        self._returns: list[int]
+        self._branches: defaultdict[nodes.LocalsDictNodeNG, int]
+        self._stmts: list[int]
 
-    def open(self):
+    def open(self) -> None:
         """Initialize visit variables."""
         self.linter.stats.reset_node_count()
         self._returns = []
         self._branches = defaultdict(int)
         self._stmts = []
-        self._exclude_too_few_public_methods = utils.get_global_option(
-            self, "exclude-too-few-public-methods", default=[]
+        self._exclude_too_few_public_methods = (
+            self.linter.config.exclude_too_few_public_methods
         )
 
-    def _inc_all_stmts(self, amount):
+    def _inc_all_stmts(self, amount: int) -> None:
         for i, _ in enumerate(self._stmts):
             self._stmts[i] += amount
 
-    @cached_property
-    def _ignored_argument_names(self):
-        return utils.get_global_option(self, "ignored-argument-names", default=None)
-
-    @check_messages(
+    @only_required_for_messages(
         "too-many-ancestors",
         "too-many-instance-attributes",
         "too-few-public-methods",
@@ -436,24 +427,25 @@ class MisdesignChecker(BaseChecker):
     def visit_classdef(self, node: nodes.ClassDef) -> None:
         """Check size of inheritance hierarchy and number of instance attributes."""
         parents = _get_parents(
-            node, STDLIB_CLASSES_IGNORE_ANCESTOR.union(self.config.ignored_parents)
+            node,
+            STDLIB_CLASSES_IGNORE_ANCESTOR.union(self.linter.config.ignored_parents),
         )
         nb_parents = len(parents)
-        if nb_parents > self.config.max_parents:
+        if nb_parents > self.linter.config.max_parents:
             self.add_message(
                 "too-many-ancestors",
                 node=node,
-                args=(nb_parents, self.config.max_parents),
+                args=(nb_parents, self.linter.config.max_parents),
             )
 
-        if len(node.instance_attrs) > self.config.max_attributes:
+        if len(node.instance_attrs) > self.linter.config.max_attributes:
             self.add_message(
                 "too-many-instance-attributes",
                 node=node,
-                args=(len(node.instance_attrs), self.config.max_attributes),
+                args=(len(node.instance_attrs), self.linter.config.max_attributes),
             )
 
-    @check_messages("too-few-public-methods", "too-many-public-methods")
+    @only_required_for_messages("too-few-public-methods", "too-many-public-methods")
     def leave_classdef(self, node: nodes.ClassDef) -> None:
         """Check number of public methods."""
         my_methods = sum(
@@ -467,11 +459,11 @@ class MisdesignChecker(BaseChecker):
         # for classes such as unittest.TestCase, which provides
         # a lot of assert methods. It doesn't make sense to warn
         # when the user subclasses TestCase to add his own tests.
-        if my_methods > self.config.max_public_methods:
+        if my_methods > self.linter.config.max_public_methods:
             self.add_message(
                 "too-many-public-methods",
                 node=node,
-                args=(my_methods, self.config.max_public_methods),
+                args=(my_methods, self.linter.config.max_public_methods),
             )
 
         # Stop here if the class is excluded via configuration.
@@ -492,14 +484,14 @@ class MisdesignChecker(BaseChecker):
         # This checks all the methods defined by ancestors and
         # by the current class.
         all_methods = _count_methods_in_class(node)
-        if all_methods < self.config.min_public_methods:
+        if all_methods < self.linter.config.min_public_methods:
             self.add_message(
                 "too-few-public-methods",
                 node=node,
-                args=(all_methods, self.config.min_public_methods),
+                args=(all_methods, self.linter.config.min_public_methods),
             )
 
-    @check_messages(
+    @only_required_for_messages(
         "too-many-return-statements",
         "too-many-branches",
         "too-many-arguments",
@@ -509,13 +501,13 @@ class MisdesignChecker(BaseChecker):
     )
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
         """Check function name, docstring, arguments, redefinition,
-        variable names, max locals
+        variable names, max locals.
         """
         # init branch and returns counters
         self._returns.append(0)
         # check number of arguments
         args = node.args.args
-        ignored_argument_names = self._ignored_argument_names
+        ignored_argument_names = self.linter.config.ignored_argument_names
         if args is not None:
             ignored_args_num = 0
             if ignored_argument_names:
@@ -524,26 +516,33 @@ class MisdesignChecker(BaseChecker):
                 )
 
             argnum = len(args) - ignored_args_num
-            if argnum > self.config.max_args:
+            if argnum > self.linter.config.max_args:
                 self.add_message(
                     "too-many-arguments",
                     node=node,
-                    args=(len(args), self.config.max_args),
+                    args=(len(args), self.linter.config.max_args),
                 )
         else:
             ignored_args_num = 0
         # check number of local variables
         locnum = len(node.locals) - ignored_args_num
-        if locnum > self.config.max_locals:
+
+        # decrement number of local variables if '_' is one of them
+        if "_" in node.locals:
+            locnum -= 1
+
+        if locnum > self.linter.config.max_locals:
             self.add_message(
-                "too-many-locals", node=node, args=(locnum, self.config.max_locals)
+                "too-many-locals",
+                node=node,
+                args=(locnum, self.linter.config.max_locals),
             )
         # init new statements counter
         self._stmts.append(1)
 
     visit_asyncfunctiondef = visit_functiondef
 
-    @check_messages(
+    @only_required_for_messages(
         "too-many-return-statements",
         "too-many-branches",
         "too-many-arguments",
@@ -552,29 +551,29 @@ class MisdesignChecker(BaseChecker):
     )
     def leave_functiondef(self, node: nodes.FunctionDef) -> None:
         """Most of the work is done here on close:
-        checks for max returns, branch, return in __init__
+        checks for max returns, branch, return in __init__.
         """
         returns = self._returns.pop()
-        if returns > self.config.max_returns:
+        if returns > self.linter.config.max_returns:
             self.add_message(
                 "too-many-return-statements",
                 node=node,
-                args=(returns, self.config.max_returns),
+                args=(returns, self.linter.config.max_returns),
             )
         branches = self._branches[node]
-        if branches > self.config.max_branches:
+        if branches > self.linter.config.max_branches:
             self.add_message(
                 "too-many-branches",
                 node=node,
-                args=(branches, self.config.max_branches),
+                args=(branches, self.linter.config.max_branches),
             )
         # check number of statements
         stmts = self._stmts.pop()
-        if stmts > self.config.max_statements:
+        if stmts > self.linter.config.max_statements:
             self.add_message(
                 "too-many-statements",
                 node=node,
-                args=(stmts, self.config.max_statements),
+                args=(stmts, self.linter.config.max_statements),
             )
 
     leave_asyncfunctiondef = leave_functiondef
@@ -587,7 +586,7 @@ class MisdesignChecker(BaseChecker):
 
     def visit_default(self, node: nodes.NodeNG) -> None:
         """Default visit method -> increments the statements counter if
-        necessary
+        necessary.
         """
         if node.is_statement:
             self._inc_all_stmts(1)
@@ -605,7 +604,7 @@ class MisdesignChecker(BaseChecker):
         self._inc_branch(node, 2)
         self._inc_all_stmts(2)
 
-    @check_messages("too-many-boolean-expressions")
+    @only_required_for_messages("too-many-boolean-expressions", "too-many-branches")
     def visit_if(self, node: nodes.If) -> None:
         """Increments the branches counter and checks boolean expressions."""
         self._check_boolean_expressions(node)
@@ -618,19 +617,19 @@ class MisdesignChecker(BaseChecker):
         self._inc_branch(node, branches)
         self._inc_all_stmts(branches)
 
-    def _check_boolean_expressions(self, node):
+    def _check_boolean_expressions(self, node: nodes.If) -> None:
         """Go through "if" node `node` and count its boolean expressions
-        if the 'if' node test is a BoolOp node
+        if the 'if' node test is a BoolOp node.
         """
         condition = node.test
         if not isinstance(condition, astroid.BoolOp):
             return
         nb_bool_expr = _count_boolean_expressions(condition)
-        if nb_bool_expr > self.config.max_bool_expr:
+        if nb_bool_expr > self.linter.config.max_bool_expr:
             self.add_message(
                 "too-many-boolean-expressions",
                 node=condition,
-                args=(nb_bool_expr, self.config.max_bool_expr),
+                args=(nb_bool_expr, self.linter.config.max_bool_expr),
             )
 
     def visit_while(self, node: nodes.While) -> None:
@@ -642,10 +641,10 @@ class MisdesignChecker(BaseChecker):
 
     visit_for = visit_while
 
-    def _inc_branch(self, node, branchesnum=1):
+    def _inc_branch(self, node: nodes.NodeNG, branchesnum: int = 1) -> None:
         """Increments the branches counter."""
         self._branches[node.scope()] += branchesnum
 
 
-def register(linter: "PyLinter") -> None:
+def register(linter: PyLinter) -> None:
     linter.register_checker(MisdesignChecker(linter))
